@@ -35,6 +35,8 @@
 //! }
 //! ```
 //!
+//! For more usage examples, please take a look at the `examples/` directory.
+//!
 //! # Feature flags
 //!
 //! - `linux-static-libusb`: uses statically linked `libusb` backend on Linux
@@ -45,6 +47,7 @@
 //! - `illumos-static-libusb`: uses statically linked `libusb` backend on Illumos (default)
 //! - `illumos-shared-libusb`: uses statically linked `hidraw` backend on Illumos
 //! - `macos-shared-device`: enables shared access to HID devices on MacOS
+//! - `windows-native`: talks to hid.dll directly without using the `hidapi` C library
 //!
 //! ## Linux backends
 //!
@@ -56,32 +59,12 @@
 //! Since `hidapi` 0.12 it is possible to open MacOS devices with shared access, so that multiple
 //! [`HidDevice`] handles can access the same physical device. For backward compatibility this is
 //! an opt-in that can be enabled with the `macos-shared-device` feature flag.
-
-extern crate libc;
-#[cfg(all(feature = "linux-native", target_os = "linux"))]
-extern crate nix;
-
-#[cfg(target_os = "windows")]
-extern crate winapi;
-#[cfg(target_os = "windows")]
-use winapi::shared::guiddef::GUID;
+#![cfg_attr(docsrs, feature(doc_cfg))]
 
 mod error;
 mod ffi;
 
-#[cfg(hidapi)]
-mod hidapi;
-
-#[cfg(all(feature = "linux-native", target_os = "linux"))]
-#[cfg_attr(docsrs, doc(cfg(all(feature = "linux-native", target_os = "linux"))))]
-mod linux_native;
-#[cfg(target_os = "macos")]
-#[cfg_attr(docsrs, doc(cfg(target_os = "macos")))]
-mod macos;
-#[cfg(target_os = "windows")]
-#[cfg_attr(docsrs, doc(cfg(target_os = "windows")))]
-mod windows;
-
+use cfg_if::cfg_if;
 use libc::wchar_t;
 use std::ffi::CStr;
 use std::ffi::CString;
@@ -91,10 +74,54 @@ use std::sync::Mutex;
 
 pub use error::HidError;
 
-#[cfg(hidapi)]
-use crate::hidapi::HidApiBackend;
-#[cfg(all(feature = "linux-native", target_os = "linux"))]
-use linux_native::HidApiBackend;
+cfg_if! {
+    if #[cfg(all(feature = "linux-native", target_os = "linux"))] {
+        //#[cfg_attr(docsrs, doc(cfg(all(feature = "linux-native", target_os = "linux"))))]
+        mod linux_native;
+        use linux_native::HidApiBackend;
+    } else if #[cfg(all(feature = "windows-native", target_os = "windows"))] {
+        //#[cfg_attr(docsrs, doc(cfg(all(feature = "windows-native", target_os = "windows"))))]
+        mod windows_native;
+        use windows_native::HidApiBackend;
+    } else if #[cfg(hidapi)] {
+        mod hidapi;
+        use hidapi::HidApiBackend;
+    } else {
+        compile_error!("No backend selected");
+    }
+}
+
+// Automatically implement the top trait
+cfg_if! {
+    if #[cfg(target_os = "windows")] {
+        #[cfg_attr(docsrs, doc(cfg(target_os = "windows")))]
+        mod windows;
+        use windows::GUID;
+        /// A trait with the extra methods that are available on Windows
+        trait HidDeviceBackendWindows {
+            /// Get the container ID for a HID device
+            fn get_container_id(&self) -> HidResult<GUID>;
+        }
+        trait HidDeviceBackend: HidDeviceBackendBase + HidDeviceBackendWindows + Send {}
+        impl<T> HidDeviceBackend for T where T: HidDeviceBackendBase + HidDeviceBackendWindows + Send {}
+    } else if #[cfg(target_os = "macos")] {
+        #[cfg_attr(docsrs, doc(cfg(target_os = "macos")))]
+        mod macos;
+        /// A trait with the extra methods that are available on macOS
+        trait HidDeviceBackendMacos {
+            /// Get the location ID for a [`HidDevice`] device.
+            fn get_location_id(&self) -> HidResult<u32>;
+
+            /// Check if the device was opened in exclusive mode.
+            fn is_open_exclusive(&self) -> HidResult<bool>;
+        }
+        trait HidDeviceBackend: HidDeviceBackendBase + HidDeviceBackendMacos + Send {}
+        impl<T> HidDeviceBackend for T where T: HidDeviceBackendBase + HidDeviceBackendMacos + Send {}
+    } else {
+        trait HidDeviceBackend: HidDeviceBackendBase + Send {}
+        impl<T> HidDeviceBackend for T where T: HidDeviceBackendBase + Send {}
+    }
+}
 
 pub type HidResult<T> = Result<T, HidError>;
 pub const MAX_REPORT_DESCRIPTOR_SIZE: usize = 4096;
@@ -168,9 +195,11 @@ impl HidApi {
     pub fn new() -> HidResult<Self> {
         lazy_init(true)?;
 
-        let device_list = HidApiBackend::get_hid_device_info_vector()?;
-
-        Ok(HidApi { device_list })
+        let mut api = HidApi {
+            device_list: Vec::with_capacity(8),
+        };
+        api.add_devices(0, 0)?;
+        Ok(api)
     }
 
     /// Create a new hidapi context, in "do not enumerate" mode.
@@ -191,13 +220,29 @@ impl HidApi {
 
     /// Refresh devices list and information about them (to access them use
     /// `device_list()` method)
+    /// Identical to `reset_devices()` followed by `add_devices(0, 0)`.
     pub fn refresh_devices(&mut self) -> HidResult<()> {
-        let device_list = HidApiBackend::get_hid_device_info_vector()?;
-        self.device_list = device_list;
+        self.reset_devices()?;
+        self.add_devices(0, 0)?;
         Ok(())
     }
 
-    /// Returns iterator containing information about attached HID devices.
+    /// Reset devices list. Intended to be used with the `add_devices` method.
+    pub fn reset_devices(&mut self) -> HidResult<()> {
+        self.device_list.clear();
+        Ok(())
+    }
+
+    /// Indexes devices that match the given VID and PID filters.
+    /// 0 indicates no filter.
+    pub fn add_devices(&mut self, vid: u16, pid: u16) -> HidResult<()> {
+        self.device_list
+            .append(&mut HidApiBackend::get_hid_device_info_vector(vid, pid)?);
+        Ok(())
+    }
+
+    /// Returns iterator containing information about attached HID devices
+    /// that have been indexed, either by `refresh_devices` or `add_devices`.
     pub fn device_list(&self) -> impl Iterator<Item = &DeviceInfo> {
         self.device_list.iter()
     }
@@ -233,7 +278,7 @@ impl HidApi {
         let device = unsafe { ffi::hid_libusb_wrap_sys_device(sys_dev, interface_num) };
 
         if device.is_null() {
-            match self.check_error() {
+            match HidApiBackend::check_error() {
                 Ok(err) => Err(err),
                 Err(e) => Err(e),
             }
@@ -258,6 +303,7 @@ impl HidApi {
     }
 }
 
+#[allow(dead_code)]
 #[derive(Clone, PartialEq)]
 enum WcharString {
     String(String),
@@ -444,42 +490,6 @@ trait HidDeviceBackendBase {
     }
 }
 
-/// A trait with the extra methods that are available on macOS
-#[cfg(target_os = "macos")]
-trait HidDeviceBackendMacos {
-    /// Get the location ID for a [`HidDevice`] device.
-    fn get_location_id(&self) -> HidResult<u32>;
-
-    /// Check if the device was opened in exclusive mode.
-    fn is_open_exclusive(&self) -> HidResult<bool>;
-}
-
-/// A trait with the extra methods that are available on macOS
-#[cfg(target_os = "windows")]
-trait HidDeviceBackendWindows {
-    /// Get the container ID for a HID device
-    fn get_container_id(&self) -> HidResult<GUID>;
-}
-
-#[cfg(not(any(target_os = "macos", target_os = "windows")))]
-trait HidDeviceBackend: HidDeviceBackendBase + Send {}
-#[cfg(target_os = "macos")]
-trait HidDeviceBackend: HidDeviceBackendBase + HidDeviceBackendMacos + Send {}
-#[cfg(target_os = "windows")]
-trait HidDeviceBackend: HidDeviceBackendBase + HidDeviceBackendWindows + Send {}
-
-/// Automatically implement the top trait
-#[cfg(not(any(target_os = "macos", target_os = "windows")))]
-impl<T> HidDeviceBackend for T where T: HidDeviceBackendBase + Send {}
-
-/// Automatically implement the top trait
-#[cfg(target_os = "macos")]
-impl<T> HidDeviceBackend for T where T: HidDeviceBackendBase + HidDeviceBackendMacos + Send {}
-
-/// Automatically implement the top trait
-#[cfg(target_os = "windows")]
-impl<T> HidDeviceBackend for T where T: HidDeviceBackendBase + HidDeviceBackendWindows + Send {}
-
 pub struct HidDevice {
     inner: Box<dyn HidDeviceBackend>,
 }
@@ -511,6 +521,8 @@ impl HidDevice {
         self.inner.check_error()
     }
 
+    /// Write an Output report to a HID device.
+    ///
     /// The first byte of `data` must contain the Report ID. For
     /// devices which only support a single report, this must be set
     /// to 0x0. The remaining bytes contain the report data. Since
@@ -523,26 +535,37 @@ impl HidDevice {
     /// `write()` will send the data on the first OUT endpoint, if
     /// one exists. If it does not, it will send the data through
     /// the Control Endpoint (Endpoint 0).
+    ///
+    /// If successful, returns the actual number of bytes written.
     pub fn write(&self, data: &[u8]) -> HidResult<usize> {
         self.inner.write(data)
     }
 
+    /// Read an Input report from a HID device.
+    ///
     /// Input reports are returned to the host through the 'INTERRUPT IN'
     /// endpoint. The first byte will contain the Report number if the device
     /// uses numbered reports.
+    ///
+    /// If successful, returns the actual number of bytes read.
     pub fn read(&self, buf: &mut [u8]) -> HidResult<usize> {
         self.inner.read(buf)
     }
 
+    /// Read an Input report from a HID device with timeout.
+    ///
     /// Input reports are returned to the host through the 'INTERRUPT IN'
     /// endpoint. The first byte will contain the Report number if the device
     /// uses numbered reports. Timeout measured in milliseconds, set -1 for
     /// blocking wait.
+    ///
+    /// If successful, returns the actual number of bytes read.
     pub fn read_timeout(&self, buf: &mut [u8], timeout: i32) -> HidResult<usize> {
         self.inner.read_timeout(buf, timeout)
     }
 
     /// Send a Feature report to the device.
+    ///
     /// Feature reports are sent over the Control endpoint as a
     /// Set_Report transfer.  The first byte of `data` must contain the
     /// 'Report ID'. For devices which only support a single report, this must
@@ -553,13 +576,20 @@ impl HidDevice {
     /// `send_feature_report()`: 'the Report ID' (or 0x0, for devices which
     /// do not use numbered reports), followed by the report data (16 bytes).
     /// In this example, the length passed in would be 17.
+    ///
+    /// If successful, returns the actual number of bytes written.
     pub fn send_feature_report(&self, data: &[u8]) -> HidResult<()> {
         self.inner.send_feature_report(data)
     }
 
+    /// Get a feature report from a HID device.
+    ///
     /// Set the first byte of `buf` to the 'Report ID' of the report to be read.
     /// Upon return, the first byte will still contain the Report ID, and the
     /// report data will start in `buf[1]`.
+    ///
+    /// If successful, returns the number of bytes read plus one for the report ID (which is still
+    /// in the first byte).
     pub fn get_feature_report(&self, buf: &mut [u8]) -> HidResult<usize> {
         self.inner.get_feature_report(buf)
     }
