@@ -8,7 +8,7 @@ use std::{
     fs::{File, OpenOptions},
     io::{Cursor, Read, Seek, SeekFrom},
     os::{
-        fd::{AsRawFd, OwnedFd},
+        fd::{AsFd, AsRawFd, BorrowedFd, OwnedFd},
         unix::{ffi::OsStringExt, fs::OpenOptionsExt},
     },
     path::{Path, PathBuf},
@@ -33,7 +33,7 @@ const BUS_SPI: u16 = 0x1C;
 pub struct HidApiBackend;
 
 impl HidApiBackend {
-    pub fn get_hid_device_info_vector() -> HidResult<Vec<DeviceInfo>> {
+    pub fn get_hid_device_info_vector(vid: u16, pid: u16) -> HidResult<Vec<DeviceInfo>> {
         // The C version assumes these can't fail, and they should only fail in case
         // of memory allocation issues, at which point maybe we should panic
         let mut enumerator = match udev::Enumerator::new() {
@@ -49,6 +49,8 @@ impl HidApiBackend {
         let devices = scan
             .filter_map(|device| device_to_hid_device_info(&device))
             .flatten()
+            .filter(|device| vid == 0 || device.vendor_id == vid)
+            .filter(|device| pid == 0 || device.product_id == pid)
             .collect::<Vec<_>>();
 
         Ok(devices)
@@ -411,7 +413,7 @@ unsafe impl Send for HidDevice {}
 // API for the library to call us, or for internal uses
 impl HidDevice {
     pub(crate) fn open(vid: u16, pid: u16, sn: Option<&str>) -> HidResult<Self> {
-        for device in HidApiBackend::get_hid_device_info_vector()?
+        for device in HidApiBackend::get_hid_device_info_vector(0, 0)?
             .iter()
             .filter(|device| device.vendor_id == vid && device.product_id == pid)
         {
@@ -471,6 +473,12 @@ impl HidDevice {
     }
 }
 
+impl AsFd for HidDevice {
+    fn as_fd(&self) -> BorrowedFd {
+        self.fd.as_fd()
+    }
+}
+
 impl HidDeviceBackendBase for HidDevice {
     fn write(&self, data: &[u8]) -> HidResult<usize> {
         if data.is_empty() {
@@ -487,7 +495,7 @@ impl HidDeviceBackendBase for HidDevice {
     }
 
     fn read_timeout(&self, buf: &mut [u8], timeout: i32) -> HidResult<usize> {
-        let pollfd = PollFd::new(self.fd.as_raw_fd(), PollFlags::POLLIN);
+        let pollfd = PollFd::new(&self.fd, PollFlags::POLLIN);
         let res = poll(&mut [pollfd], timeout)?;
 
         if res == 0 {
