@@ -1,6 +1,5 @@
-use winapi::shared::guiddef::GUID;
-
 use crate::{HidDevice, HidResult};
+pub use windows_sys::core::GUID;
 
 impl HidDevice {
     /// Get the container ID for a HID device.
